@@ -1,3 +1,4 @@
+# ignore this first chunk of comments if you dont wanna hear me rant ;)
 # so you wanna know whats going on in this file eh
 # basically these are all functions the auth module uses to manage ya things
 # because this is my file and im not regulated by sum community
@@ -37,11 +38,11 @@ import time
 import threading
 import logging
 import traceback
+import asyncio
 
 from datetime import datetime
-from flask_sqlalchemy import SQLAlchemy
-from flask import Flask
-from flask_restful import fields, marshal
+from sqlalchemy import create_engine, Column, String, Table, MetaData
+from sqlalchemy.pool import StaticPool
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
@@ -52,15 +53,32 @@ from cryptography.fernet import Fernet, InvalidToken
 # this lil john here is just used to get a file path to the folder where the module is stored
 # for logging purposes, it just makes more sense to me to have logs stored in the module folder
 # instead of the cwd, but i also made it user defined if you so choose
-from . import logs
+from .logs.log_class import Logger
+
 
 # these are all the exceptions that this module will raise
 # i could use like built in ones but ion feel like it so...
-class LocationError(BaseException): ...
-class AuthenticationError(BaseException): ...
-class UsernameError(AuthenticationError): ...
-class PasswordError(AuthenticationError): ...
-class DataError(BaseException): ...
+class AuthError(Exception): ...
+class LocationError(AuthError): ...
+class AuthenticationError(AuthError): ...
+class UsernameError(AuthError): ...
+class PasswordError(AuthError): ...
+class DataError(AuthError): ...
+
+old_hook = sys.excepthook
+
+def exception_hook(exc_type, value, tb):
+    tb = traceback.format_exception(exc_type, value=value, tb=tb)
+    if exc_type.__bases__[0] == AuthError:
+        bottom = tb[-1:]
+        tb = tb[:-3]
+        tb.append(bottom[0].strip('sixauth.main.'))
+        tb = ''.join(tb)
+        print(tb)
+    else:
+        old_hook(exc_type, value, tb)
+    
+sys.excepthook = exception_hook
 
 # alr alr so first real thing here is this cache dict
 # so this john will hold all the active users data for way quicker access
@@ -72,51 +90,24 @@ class DataError(BaseException): ...
 # i lowk dont like just creating this thing like this but ion know a better way with out classes and whatnot
 cache = {}
 
-# here we begin to set up the loggers
-# i have four different loggers, one for console 
-# and one for debugging, then the same thing for the client side
-# i want to change the client console logger to be
-# a more indepth server logger and then reduce the amount of things
-# logged by the server, cause rn if you have like 100 clients connect to the server and 
-# manipulate data, log files get into he gigabytes of size
-# im actually gonna do that now which makes all of what i said irrelevant 
-# but atleast you get to see my thought prosses
-# ok but here we define the names, levels, and formats of the loggers
+# here we are setting up the loggers to be passed to the logging module
+# get level and name and format, all the fun stuff
 server_console = logging.getLogger('server_console')
+client_console = logging.getLogger('client_console')
 server_logger = logging.getLogger('server_logger')
-server_big_logger = logging.getLogger('server_big_logger')
 client_logger = logging.getLogger('client_logger')
 server_console.setLevel(logging.INFO)
+client_console.setLevel(logging.INFO)
 server_logger.setLevel(logging.INFO)
-server_big_logger.setLevel(logging.INFO)
 client_logger.setLevel(logging.INFO)
 console_handler = logging.StreamHandler()
 formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
 
-# here we define the function that sets up the loggers and their default states
-# we set things like log file paths and how much to log, the only changable things
-# we also log that we have started a newlog and what handlers go to what loggers
-def log_paths(client_logger_location = os.path.dirname(logs.__file__), server_logger_location = os.getcwd(), inDepthLogs = False): 
-    global server_logger_handler
-    global client_logger_handler
-    server_logger_handler = logging.FileHandler(server_logger_location+'/server.log')
-    client_logger_handler = logging.FileHandler(client_logger_location+'/client.log')
-    server_console.addHandler(server_logger_handler)
-    server_console.addHandler(console_handler)
-    server_logger.addHandler(server_logger_handler)
-    if inDepthLogs:
-        server_big_logger.addHandler(server_logger_handler)
-    client_logger.addHandler(client_logger_handler)
-    server_logger.info('VVV---------BEGIN-NEW-LOG----------VVV')
-    client_logger.info('VVV---------BEGIN-NEW-LOG----------VVV')
-    server_logger_handler.setFormatter(formatter)
-    client_logger_handler.setFormatter(formatter)
-
 # and here we just run the defult state so that we are always logging something
-# we can run the function above at anytime during runtime to change this
 # it will show in the logs that a new log has started
 # and from then on all the loggers will have the new paths and states
-log_paths()
+# you can from main import logger and then run the setup_logger method to change how the logger logs things
+logger = Logger(server_console, client_console, server_logger, client_logger, console_handler, formatter).setup_logger(server_logger_location=None)
 
 # now for the first big function here
 # this is the cache check loop that we run in a separate thread
@@ -132,13 +123,12 @@ log_paths()
 # if the time is past a defined threshold, then we delete it and declare in the log that a user timed out
 # so save resources we only run this chack once a second
 # also the stop flag stops this thread from the main thread
+@logger(is_log_more=True)
 def check_and_remove(threshold, stop_flag):
     while not stop_flag.is_set():
         for key in list(cache):
             if time.time() - cache[key]['time'] > threshold:
                 del cache[key]
-                server_logger.info('A user timed out')
-                server_big_logger.info(f'Current cache: {cache}')
         time.sleep(1)
 
 # the only known issue with the checker is that
@@ -157,6 +147,7 @@ def check_and_remove(threshold, stop_flag):
 # then when the thread joines back to the keep alive thread
 # it will loop and start a new checker thread
 # should also mention that this is run in its own thread much like the checker is
+@logger(is_log_more=True)
 def keep_alive(cache_threshold, stop_flag):
     while not stop_flag.is_set():
         t = threading.Thread(target=check_and_remove, args=(cache_threshold, stop_flag))
@@ -175,7 +166,8 @@ def keep_alive(cache_threshold, stop_flag):
 # then we define a key driver object and derive the key from the password
 # then create a fernet object with with the key and then encrypt the data 
 # with the fernet object, and then return it, easy!
-def encrypt_data(data, password, username):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def encrypt_data(data, username, password):
     json_data = json.dumps(data)
     kdf = PBKDF2HMAC(
         algorithm=hashes.SHA256(),
@@ -195,7 +187,8 @@ def encrypt_data(data, password, username):
 # return that dict and easy money
 # for the both of these functions tho, i dont know if they are the best way to do this
 # its the way we have and untill told otherwise im not changing it lol
-def decrypt_data(data, password, username):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def decrypt_data(data, username, password):
     kdf = PBKDF2HMAC(
         algorithm=hashes.SHA256(),
         length=32,
@@ -210,6 +203,7 @@ def decrypt_data(data, password, username):
 # just have the message and the key
 # very quickly do everything that the above functions do 
 # except for derive the key
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def encrypt_data_fast(message, key):
     return Fernet(bytes.fromhex(key)).encrypt(json.dumps(message).encode())
 
@@ -217,6 +211,7 @@ def encrypt_data_fast(message, key):
 # this is way less secure and doesnt last any longer than one run of the client
 # as the key is never stored on the client outside of the client script
 # but because the data is not stored in a file this way it shouldnt need to be too secure i think
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def decrypt_data_fast(message, key):
     return json.loads(Fernet(bytes.fromhex(key)).decrypt(message).decode())
 
@@ -226,6 +221,7 @@ def decrypt_data_fast(message, key):
 # i realized i should move these outside of the functions to one place
 # as these are ever changing as hardware and software change
 # will be easier to update these when better hashing algorithms come out
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def create_password_hash(password):
     return bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt()).hex()
 
@@ -233,6 +229,7 @@ def create_password_hash(password):
 # and one of the easiest ones to use
 # just hash the john then check if the password and the hash match
 # obviously more than that happens, but this isnt a course on cryptograghy lol
+@logger(is_log_more=True, in_sensitive=True)
 def verify_password_hash(hash, password):
     return bcrypt.checkpw(password.encode('utf-8'), bytes.fromhex(hash))
 
@@ -241,14 +238,16 @@ def verify_password_hash(hash, password):
 # and we use it because the jsonpath_ng module doesnt allow numbers in the 
 # parser configuration, just do this regular conversion and the end user never knows we did it
 # unless they look at the raw dict lol
-def convert_numbers_to_words(text):
-        return text.replace('1', 'one').replace('2', 'two').replace('3', 'three').replace('4', 'four').replace('5', 'five').replace('6', 'six').replace('7', 'seven').replace('8', 'eight').replace('9', 'nine').replace('0', 'zero')
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def convert_numbers_to_words(data):
+        return data.replace('1', 'one').replace('2', 'two').replace('3', 'three').replace('4', 'four').replace('5', 'five').replace('6', 'six').replace('7', 'seven').replace('8', 'eight').replace('9', 'nine').replace('0', 'zero')
 
 # this lil john just checks if an object 
 # is json serialized
 # we do this because we make this check alot and i dont like
 # try and except statements in like main code ya know
 # rather have it in a dedicated function
+@logger(is_log_more=True, in_sensitive=True)
 def is_json_serialized(obj):
     try:
         json.loads(obj)
@@ -261,12 +260,12 @@ def is_json_serialized(obj):
 # just moved it to its own function
 # we use this in the cache functions to check if the key provided by the client works for 
 # the hash they are trying to access, if not, boot them
+@logger(is_log_more=True, in_sensitive=True)
 def is_valid_key(data, id):
     try:
         decrypt_data_fast(data, id)
         return True
     except InvalidToken:
-        server_big_logger.info(f'Key invalid, {id}')
         return False
 
 # now this is a big one lol
@@ -281,7 +280,8 @@ def is_valid_key(data, id):
 # we also return the socket connection with the server so you can send and recive messages
 # actually not that big ngl, just alot of data moving to securly get a shared key
 # this code was half created by the chatGPT bot, was using ssl before this and was having trouble with certs
-def establish_connection(address):
+@logger(is_log_more=True)
+def establish_client_connection(address):
     client_private_key = ec.generate_private_key(ec.SECP384R1, default_backend())
     client_public_key = client_private_key.public_key()
     client_public_key_bytes = client_public_key.public_bytes(
@@ -312,10 +312,10 @@ def establish_connection(address):
 # we also log what is in the cache if we are logging big things
 # oh and this returns the a dict with a succsess code and pointer, i call it hash just because its the hash of the id plus datetime as a way of ensuring 
 # no two hashes are the same
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def add_user(id):
     hash = hashlib.sha512((f'{id}{datetime.now()}').encode("UTF-8")).hexdigest()
     cache[hash] = {'main':encrypt_data_fast([None,(None,None)],id), 'time':time.time()}
-    server_big_logger.info(f'Current cache: {cache}')
     return {'code':200, 'hash':hash}
 
 # this john here returns the data in the cache for a user
@@ -330,11 +330,11 @@ def add_user(id):
 # then that code will no longer throw, also when the user logs out, the info in the cache
 # is set the same way as when their first added, so [None,(None,None)]
 # the default state of the cache
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def find_user(hash, id):
     if not is_valid_key(cache[hash]['main'], id):
         return {'code':500}
     cache[hash]['time'] = time.time()
-    server_big_logger.info(f'Current cache: {cache}')
     data = decrypt_data_fast(cache[hash]['main'],id)
     if data[0] == None:
         return {'code':500}
@@ -345,21 +345,21 @@ def find_user(hash, id):
 # the data in the cache with it
 # all the cache functions are really easy
 # and thats why its so fast
+@logger(is_log_more=True, in_sensitive=True)
 def update_user(hash, id, dbdat):
     if is_valid_key(cache[hash]['main'], id):
         cache[hash]['main'] = encrypt_data_fast(dbdat,id)
         cache[hash]['time'] = time.time()
-        server_big_logger.info(f'Current cache: {cache}')
         return {'code':200}
     return {'code':500}
 
 # and this last john just removes the users cache
 # if we pass that key check again
 # oh yeah also we log the changes in the cache
+@logger(is_log_more=True, in_sensitive=True)
 def delete_user(hash, id):
     if is_valid_key(cache[hash]['main'], id):
         del cache[hash]
-        server_big_logger.info(f'Current cache: {cache}')
         return {'code':200}
     return {'code':500}
 
@@ -370,18 +370,16 @@ def delete_user(hash, id):
 # so first we check that the username is a valid username
 # then we check that the user doesnt exsist in the database already
 # if all checks pass, then we create the users account and return a success code
-def sign_up(app, db, User, **data):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def sign_up(conn, users, **data):
     if data['username'] == '':
         return {'code':406}
     if data['username'].isalnum() == False:
         return {'code':406}
-    with app.app_context():
-        user_from_database = User.query.filter_by(username=data['username']).first()
+    user_from_database = conn.execute(users.select().where(users.c.username == data['username'])).fetchone()
     if user_from_database:
         return {'code':409}
-    with app.app_context():
-        db.session.add(User(username=data['username'], password=create_password_hash(data['password']), data=encrypt_data({}, data['username'], data['password'])))
-        db.session.commit()
+    conn.execute(users.insert().values(username=data['username'], password=create_password_hash(data['password']), data=encrypt_data({}, data['username'], data['password'])))
     return {'code':200}
 
 # save data is pretty simple as well
@@ -394,6 +392,7 @@ def sign_up(app, db, User, **data):
 # if we dont do that then we will use jsonpath_ng to parse the dict of users data and then
 # update or create the data into the location
 # send this updated dict back to the cache and then return the function with 200
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def save_data(**data):
     user_from_cache = find_user(data['hash'], data['id'])
     if user_from_cache['code'] == 500:
@@ -415,6 +414,7 @@ def save_data(**data):
 # otherwise we use jsonpath_ng to run through the dict of users data and then delete what data it finds
 # oh and if jsonpath_ng cant find the location then we return an error code for that
 # then we put the updated dict back into the cache and return the success code
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def delete_data(**data):
     user_from_cache = find_user(data['hash'], data['id'])
     if user_from_cache['code'] == 500:
@@ -438,25 +438,17 @@ def delete_data(**data):
 # then we chack that the password in the cache is the same as in the database
 # then we update the database with the data in the cache
 # we clear the data in the cache and return a succsess code
-def log_out(app, db, passfields, User, **data):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def log_out(conn, users, **data):
     user_from_cache = find_user(data['hash'], data['id'])
     if user_from_cache['code'] == 500:
         return {'code':200}
-    with app.app_context():
-        user_from_database = User.query.filter_by(username=user_from_cache['data'][1][0]).first()
+    user_from_database = conn.execute(users.select().where(users.c.username == data['username'])).fetchone()
     if not user_from_database:
         return {'code':420, 'data':user_from_cache['data'], 'error':'could not find user to logout'}
-    datPass = marshal(user_from_database, passfields)['password']
-    if not verify_password_hash(datPass, user_from_cache['data'][1][1]):
+    if not verify_password_hash(user_from_database[1], password=user_from_cache['data'][1][1]):
         return {'code': 423}
-    with app.app_context():
-        db.session.delete(user_from_database)
-        db.session.add(User(username=user_from_cache['data'][1][0], 
-                            password=create_password_hash(user_from_cache['data'][1][1]), 
-                            data=encrypt_data(user_from_cache['data'][0], 
-                                              user_from_cache['data'][1][0], 
-                                              user_from_cache['data'][1][1])))
-        db.session.commit()
+    conn.execute(users.update().where(users.c.username == user_from_cache['data'][1][0]).values(data=encrypt_data(user_from_cache['data'][0], user_from_cache['data'][1][0], user_from_cache['data'][1][1])))
     update_user(data['hash'], data['id'], [None,(None,None)])
     return {'code':200}
 
@@ -465,20 +457,17 @@ def log_out(app, db, passfields, User, **data):
 # run cross checks on database data and cache data
 # then remove the user from the database
 # and return success code
-def remove_account(app, db, passfields, User, **data):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def remove_account(conn, users, **data):
     user_from_cache = find_user(data['hash'], data['id'])
     if user_from_cache['code'] == 500:
         return{'code':423}
-    with app.app_context():
-        user_from_database = User.query.filter_by(username=user_from_cache['data'][1][0]).first()
+    user_from_database = conn.execute(users.select().where(users.c.username == user_from_cache['data'][1][0])).fetchone()
     if not user_from_database:
         return {'code':423}
-    datPass = marshal(user_from_database, passfields)['password']
-    if not verify_password_hash(datPass, user_from_cache['data'][1][1]):
+    if not verify_password_hash(user_from_database[1], password=user_from_cache['data'][1][1]):
         return {'code':423}
-    with app.app_context():
-        db.session.delete(user_from_database)
-        db.session.commit()
+    conn.execute(users.delete().where(users.c.username == user_from_cache['data'][1][0]))
     update_user(data['hash'], data['id'], [None,(None,None)])
     return {'code':200}
 
@@ -488,19 +477,18 @@ def remove_account(app, db, passfields, User, **data):
 # good username and password and that they match with the database
 # then return a success code
 # nothing crazy, but this is the only function that can load userdata from database to cache
-def log_in(app, datfields, passfields, User, **data):
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
+def log_in(conn, users, **data):
     if data['username'] == '':
         return {'code':406}
     if data['username'].isalnum() == False:
         return {'code':406}
-    with app.app_context():
-        user_from_database = User.query.filter_by(username=data['username']).first()
+    user_from_database = conn.execute(users.select().where(users.c.username == data['username'])).fetchone()
     if not user_from_database:
         return {'code':404}
-    datPass = marshal(user_from_database, passfields)['password']
-    if not verify_password_hash(datPass, data['password']):
+    if not verify_password_hash(user_from_database[1], password=data['password']):
         return {'code':401}   
-    cache_data = [decrypt_data(marshal(user_from_database, datfields)['data'], data['username'], data['password']), (data['username'], data['password'])]
+    cache_data = [decrypt_data(user_from_database[2], data['username'], data['password']), (data['username'], data['password'])]
     if update_user(data['hash'], data['id'], cache_data)['code'] == 500:
         return {'code':423}
     return {'code':200}
@@ -511,6 +499,7 @@ def log_in(app, datfields, passfields, User, **data):
 # kidding, just do more checks 
 # then use jsonpath_ng to retrive the data
 # the return 202 to tell the request handler that we also have data coming back to it
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def load_data(**data):
     user_from_cache = find_user(data['hash'], data['id'])
     if user_from_cache['code'] == 500:
@@ -526,13 +515,15 @@ def load_data(**data):
 # this john just makes a position in the cache for the client
 # then the any users on the client use that cache position for data storage
 # we also return the cache pointer for the client to use
+@logger(is_log_more=True, in_sensitive=True, out_sensitive=True)
 def create_session(**data):
     user_hash = add_user(data['id'])['hash']
-    return {'code':101, 'hash':user_hash}
+    return {'code':201, 'hash':user_hash}
 
 # and the end session function does the opposite
 # if finds the pointer and checks itd validity and then deletes it!
 # easy money if i do say so myself
+@logger(is_log_more=True, in_sensitive=True)
 def end_session(**data):
     if delete_user(data['hash'], data['id'])['code'] == 500:
         return {'code':423}
@@ -541,78 +532,123 @@ def end_session(**data):
 # oo the backend session john
 # this mane is called and sets up a connection with the server
 # then it returns a funtion that can send a recive data from the client
+@logger()
 def backend_session(address):
-    f, client_socket = establish_connection(address)
+    f, client_socket = establish_client_connection(address)
     client_logger.info(f'Connected to: {address}')
-    def send(**data):
+    @logger(in_sensitive=True, out_sensitive=True)
+    def session(**data):
         client_socket.send(f.encrypt(json.dumps(data).encode('utf-8')))
         return json.loads(f.decrypt(client_socket.recv(1024)).decode())
-    return send
+    return session
 
 # im lowk getting lazy writing these and would love encouragement to do better lol
 # enewaz this is the frontend session
 # it starts up a database connection and then returns a function that wraps all the above functions
+@logger()
 def frontend_session(path = os.getcwd(), test_mode = False):
-    app = Flask(__name__)
-    app.config['SQLALCHEMY_DATABASE_URI'] = f'sqlite:///{path}/database.db'
-    client_logger.info(f'Database located at: sqlite:\\\\\\{path}\\database.db')
-    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
-    db = SQLAlchemy(app)            
-    class User(db.Model):
-        username = db.Column(db.String, nullable=False, primary_key = True)
-        password = db.Column(db.String, nullable=False)
-        data = db.Column(db.String)
-        def __init__(self, username, password, data):
-            self.username = username
-            self.password = password
-            self.data = data
-    with app.app_context():
-        db.create_all()
-    datfields = {'data': fields.Raw}
-    passfields = {'password': fields.String}
-    def action(**data):
-        if data['func'] == 'create_session':
+    db_path = f'sqlite:///{path}/database.db'
+    client_logger.info(f'Database located at: {db_path}')
+    engine = create_engine(db_path, connect_args={'check_same_thread':False}, poolclass=StaticPool)
+    metadata = MetaData()
+    users = Table('users', metadata,
+        Column('username', String, unique=True, primary_key=True),
+        Column('password', String),
+        Column('data', String))
+    metadata.create_all(engine)
+    conn = engine.connect()
+    @logger(in_sensitive=True, out_sensitive=True)
+    def session(**data):
+        if data['code'] == 301:
             return create_session(**data)
-        elif data['func'] == 'sign_up':
-            return sign_up(app, db, User, **data)
-        elif data['func'] == 'save_data':
+        elif data['code'] == 302:
+            return sign_up(conn, users, **data)
+        elif data['code'] == 303:
             return save_data(**data)
-        elif data['func'] == 'delete_data':
+        elif data['code'] == 304:
             return delete_data(**data)
-        elif data['func'] == 'log_out':
-            return log_out(app, db, passfields, User, **data)
-        elif data['func'] == 'remove_account':
-            return remove_account(app, db, passfields, User, **data)
-        elif data['func'] == 'log_in':
-            return log_in(app, datfields, passfields, User, **data)
-        elif data['func'] == 'load_data':
+        elif data['code'] == 305:
+            return log_out(conn, users, **data)
+        elif data['code'] == 306:
+            return remove_account(conn, users, **data)
+        elif data['code'] == 307:
+            return log_in(conn, users, **data)
+        elif data['code'] == 308:
             return load_data(**data)
-        elif data['func'] == 'end_session':
+        elif data['code'] == 309:
             return end_session(**data)
-        elif data['func'] == 'test':
+        elif data['code'] == 'test':
             if test_mode:
                 return {'code':200, 'data':data}
-    return action
+    return session
 
-# and lastly the server function
-# this is by far the biggest function
-# this john will start off by setting up logging for the server
-# then it will create a socket connection on the given address and port
-# we also make some threads for cache checking and client handling
-# then we define the client function, which will be used in a thread once set up
-# the client thread will wait for a request from the client and basically just pass it to its own front end session
-# no need to do anything crazy, however we do have special cases for ensuring clients are ended properly
-# then we have the main loop that waits for a new client connection and sets up an encrypted connection
-# once encrytption is done we pass the client to its own handling thread
-# and thats it we are done!
-def server(host, port, cache_threshold = 300, debug = False, log_senseitive_info = False, test_mode = False):
-    if debug:
-        server_logger.addHandler(console_handler)
-        client_logger.addHandler(console_handler)
-        server_console.info('Debug mode active')  
-    if log_senseitive_info:
-        server_console.info('WARNING: SENSEITIVE INFORMATION BEING LOGGED')
-    client_logger.addHandler(server_logger_handler)
+# this is the start of the server functions
+# the main client loop will receive all connections from the client after encryption is setup
+# it will decrypt and parse the request and then give it to the servers frontend session
+# then it will encrypt the result and send it back
+@logger(is_log_more=True, is_server=True, in_sensitive=True)
+async def main_client_loop(client_socket, client_address, f, loop, session, stop_flag1):
+    while not stop_flag1.is_set():
+        recv = await loop.sock_recv(client_socket, 1024)
+        if recv == b'':
+            break
+        try:
+            data = json.loads(f.decrypt(recv).decode())
+            server_logger.info(f'{client_address} made request: {data["code"]}')
+            response = session(**data)
+            server_logger.info(f'response to {client_address}: {response["code"]}')
+            await loop.sock_sendall(client_socket, f.encrypt(json.dumps(response).encode('utf-8')))
+            if data['code'] == 309 and response['code'] == 200:
+                break
+        except BaseException as err:
+            if type(err) == KeyError:
+                await loop.sock_sendall(client_socket, f.encrypt(json.dumps({'code':420, 'data':None, 'error':f'Couldnt find user in cache, contact owner to recover any data, \nuse this key: {str(err)}\nuse this id: \'{str(data["id"])}\''}).encode('utf-8')))
+            else:
+                await loop.sock_sendall(client_socket, f.encrypt(json.dumps({'code':420, 'data':None, 'error':str(err)}).encode('utf-8')))
+            tb = traceback.extract_tb(sys.exc_info()[2])
+            line_number = tb[-1][1]
+            server_logger.info(f'Request prossesing for {client_address} failed, Error on line {line_number}: {str(type(err))}:{str(err)}')#\n{str(tb)}')
+
+# the setup_client function will be called for every client that connects
+# it will set up encryption and run the client main loop
+@logger(is_server=True, in_sensitive=True)
+async def setup_client(client_socket, client_address, server_public_key_bytes, stop_flag1, loop, session, server_private_key=None):
+    client_public_key_bytes = await loop.sock_recv(client_socket, 1024)
+    client_public_key = serialization.load_pem_public_key(
+    client_public_key_bytes, default_backend())
+    await loop.sock_sendall(client_socket, server_public_key_bytes)
+    shared_secret = server_private_key.exchange(ec.ECDH(), client_public_key)
+    kdf = HKDF(
+    algorithm=hashes.SHA256(),
+    length=32,
+    salt=None,
+    info=b"session key",
+    backend=default_backend())
+    key = kdf.derive(shared_secret)
+    f = Fernet(base64.urlsafe_b64encode(key))
+    await main_client_loop(client_socket, client_address, f, loop, session, stop_flag1)
+    client_socket.close()
+
+# the server main loop function will accept all new clients
+# then create an async task for the client talk to
+# then logs the task into a set too keep it alive and keep a running list of clients 
+@logger(is_server=True, in_sensitive=True)
+async def server_main_loop(server_socket, server_public_key_bytes, stop_flag1, session, server_private_key=None):
+    loop = asyncio.get_event_loop()
+    clients = set()
+    while True:
+        client_socket, client_address = await loop.sock_accept(server_socket)
+        task = asyncio.create_task(setup_client(client_socket, client_address, server_public_key_bytes, stop_flag1, loop, session, server_private_key=server_private_key))
+        clients.add(task)
+        task.add_done_callback(clients.discard)
+
+# and finally the server function is called by the end user to creat a server
+# it will set up everything the server needs to run
+# then run the main loop of the server
+@logger(is_server=True)
+def server(host, port, cache_threshold = 300, test_mode = False, use_default_logger = True):
+    if use_default_logger:
+        logger.setup_logger(client_logger_location=os.getcwd())
     session = frontend_session(test_mode=test_mode)
     stop_flag1 = threading.Event()
     t = threading.Thread(target=keep_alive, args=(cache_threshold, stop_flag1))
@@ -622,15 +658,6 @@ def server(host, port, cache_threshold = 300, debug = False, log_senseitive_info
     server_public_key_bytes = server_public_key.public_bytes(
     encoding=serialization.Encoding.PEM,
     format=serialization.PublicFormat.SubjectPublicKeyInfo)
-    clients = []
-    def exit():
-        stop_flag1.set()
-        t.join()
-        server_logger.info('Cache thread exited')
-        for client_s, client_t in clients:
-            client_s.close()
-            client_t.join()
-        server_logger.info('All client threads exited')
     server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     server_socket.setblocking(0)
     server_socket.bind((host, port))
@@ -638,70 +665,12 @@ def server(host, port, cache_threshold = 300, debug = False, log_senseitive_info
     server_console.info('Server started')
     server_console.info('Press Ctrl+C to exit')
     server_console.info(f"Listening for incoming connections on {host}:{port}")
-# i hate how nested this is...
-    def handle_client(client_socket, f, client_address, session, stop_flag):
-        try:
-            while not stop_flag.is_set():
-                try:
-                    recv = client_socket.recv(1024)
-                    server_logger.info(f"Received data from client: {client_address}: {recv}")
-                    if recv != b'':
-                        try:
-                            data = json.loads(f.decrypt(recv).decode())
-                            if log_senseitive_info:
-                                server_logger.info(f"Received: {data}")
-                            elif data['func'] != 'sign_up' and data['func'] != 'create_session':
-                                server_logger.info(f"Received: {data['func']}, {data['hash']}")
-                            response = session(**data)
-                            server_logger.info(f'Response: {response["code"]}')
-                            client_socket.send(f.encrypt(json.dumps(response).encode('utf-8')))
-                            if data['func'] == 'end_session' and response['code'] == 200:
-                                break
-                        except BaseException as err:
-                            if type(err) == KeyError:
-                                client_socket.send(f.encrypt(json.dumps({'code':420, 'data':None, 'error':f'Couldnt find user in cache, contact owner to recover any data, \nuse this key: {str(err)}\nuse this id: \'{str(data["id"])}\''}).encode('utf-8')))
-                            else:
-                                client_socket.send(f.encrypt(json.dumps({'code':420, 'data':None, 'error':str(err)}).encode('utf-8')))
-                            tb = traceback.extract_tb(sys.exc_info()[2])
-                            line_number = tb[-1][1]
-                            server_logger.info(f'Request prossesing for {client_address} failed, Error on line {line_number}: {str(type(err))}:{str(err)}\n{str(tb)}')
-                            break
-                    else:
-                        break
-                except BlockingIOError:
-                    pass
-        except BaseException as err:
-            server_console.info(f'Client thread did not exit successfully, Error: {err}')
-        server_logger.info(f"Closed connection from {client_address}")
-        client_socket.close()
-        del [client for client in clients if client[0] == client_socket][0]
     try:
-        while not stop_flag1.is_set():
-            try:
-                client_socket, client_address = server_socket.accept()
-                client_public_key_bytes = client_socket.recv(1024)
-                client_public_key = serialization.load_pem_public_key(
-                client_public_key_bytes, default_backend())
-                client_socket.send(server_public_key_bytes)
-                shared_secret = server_private_key.exchange(ec.ECDH(), client_public_key)
-                kdf = HKDF(
-                algorithm=hashes.SHA256(),
-                length=32,
-                salt=None,
-                info=b"session key",
-                backend=default_backend())
-                key = kdf.derive(shared_secret)
-                f = Fernet(base64.urlsafe_b64encode(key))
-                server_logger.info(f"Received incoming connection from {client_address}")
-                client_thread = threading.Thread(target=handle_client, args=(client_socket, f, client_address, session, stop_flag1))
-                client_thread.start()
-                server_logger.info('Client thread started')
-                clients.append((client_socket, client_thread))
-            except BlockingIOError:
-                pass
+        asyncio.run(server_main_loop(server_socket, server_public_key_bytes, stop_flag1, session, server_private_key=server_private_key))
     except KeyboardInterrupt:
-        exit()
-        server_console.info('Exited')
+        server_console.info('Server Closed')
     except BaseException as err:
-        exit()
-        server_console.info(f'Program did not exit successfully, Error: {err}')
+        server_console.info(f'Server did not exit successfully, Error: {err}')
+    finally:
+        stop_flag1.set()
+        t.join()
\ No newline at end of file
